from django.contrib import admin
from django.db.models import Max
from enumfields.admin import EnumFieldListFilter
from arkindex.dataimport.models import DataImport, DataFile, Repository, Revision
from arkindex.images.models import Image


class DataFileInline(admin.StackedInline):
    model = DataImport.files.through
    extra = 0


class ImageInline(admin.TabularInline):
    model = Image
    extra = 1


class DataImportAdmin(admin.ModelAdmin):
    list_display = ('id', 'creator', 'corpus', 'state', 'mode')
    list_filter = [('mode', EnumFieldListFilter), ]
    fields = ('id', 'creator', 'corpus', 'state', 'mode', 'payload', 'workflow')
    readonly_fields = ('id', 'workflow', 'state')
    inlines = [DataFileInline, ]

    def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs) \
            .prefetch_related('workflow__tasks') \
            .annotate(last_run=Max('workflow__tasks__run'))


class DataFileAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'corpus')
    list_filter = ('corpus', )
    fields = ('id', 'name', 'size', 'hash', 'content_type', 'corpus')
    readonly_fields = ('id', 'size', 'hash', )
    inlines = [DataFileInline, ImageInline]


class RevisionInline(admin.StackedInline):
    model = Revision


class RepositoryAdmin(admin.ModelAdmin):
    list_display = ('id', 'url', 'corpus')
    list_filter = ('corpus', )
    fields = ('id', 'url', 'corpus', 'hook_token')
    readonly_fields = ('id', )
    inlines = [RevisionInline, ]


admin.site.register(DataImport, DataImportAdmin)
admin.site.register(DataFile, DataFileAdmin)
admin.site.register(Repository, RepositoryAdmin)
