#!/usr/bin/env python3
from django.conf import settings
from ponos.management.base import PonosCommand, FolderArgument
from arkindex_common.ml_tool import MLToolType
from arkindex.images.importer import IndexImporter
from arkindex.project.argparse import CorpusArgument, DataSourceArgument
import logging

logging.basicConfig(
    format='[%(levelname)s] %(message)s',
    level=logging.INFO,
)


class Command(PonosCommand):
    help = 'Import annotations from index files (.idx.gz) as async task'
    docker_image = settings.ARKINDEX_APP_IMAGE
    base_recipe = settings.PONOS_RECIPE

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'index_folder',
            help='Folder holding index files, organized in subfolders for each volume and named after folios',
            type=FolderArgument(),
        )
        parser.add_argument(
            '--corpus',
            help='ID or part of the name of the corpus to import annotations in',
            type=CorpusArgument(),
            required=True,
        )
        parser.add_argument(
            '--source',
            help='ID or part of the slug of a Recognizer DataSource to link annotations to',
            type=DataSourceArgument(type=MLToolType.Recognizer),
            required=True,
        )
        parser.add_argument(
            '--dry-run',
            help='Only show the associated images found for all index files, without importing anything',
            action='store_true',
            default=False,
        )

    def run(self, index_folder=None, corpus=None, source=None, dry_run=False, **kwargs):
        importer = IndexImporter(index_folder, corpus=corpus, source=source)
        if dry_run:
            importer.dry_run()
        else:
            importer.run()
