# Generated by Django 2.1 on 2019-05-23 14:53

from django.db import migrations
from django.db.models import Q, OuterRef, Subquery
from arkindex_common.enums import MetaType, EntityType


def createEntities(apps, schema_editor):
    values = set(item.value for item in EntityType)
    MetaData = apps.get_model('documents', 'MetaData')
    Entity = apps.get_model('documents', 'Entity')
    for metadata in MetaData.objects \
            .filter(Q(type=MetaType.Text) & (Q(name__in=values) | Q(name='place'))) \
            .prefetch_related('element'):
        if metadata.name in values:
            type = EntityType(metadata.name)
        else:
            type = EntityType.Location
        entity, created = Entity.objects.get_or_create(
            name=metadata.value,
            type=type,
            corpus_id=metadata.element.corpus_id)
        if created:
            entity.save()
            print(entity)
        metadata.value = ''
        metadata.type = MetaType.Entity
        metadata.entity = entity
        metadata.save()


def deleteEntities(apps, schema_editor):
    MetaData = apps.get_model('documents', 'MetaData')
    # Use Subquery() because Django doesn't support F() with joins
    MetaData.objects \
        .filter(entity__isnull=False) \
        .update(
            type=MetaType.Text,
            value=Subquery(MetaData.objects.filter(pk=OuterRef('pk')).values('entity__name')[:1]))
    Entity = apps.get_model('documents', 'Entity')
    Entity.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0003_entities'),
    ]

    operations = [
        migrations.RunPython(createEntities, deleteEntities),
    ]
