from arkindex_common.ml_tool import MLToolType
from arkindex.documents.models import Element, ElementType, DataSource
from arkindex.images.importer import IndexImporter, GallicaIndexImporter
from arkindex.dataimport.iiif import IIIFParser
import logging
import os

logger = logging.getLogger(__name__)


def import_manifest(*args, **kwargs):
    """
    Import a IIIF manifest or collection from a local file or a URL using IIIFParser.
    All arguments are directly passed to IIIFParser's constructor
    except build_thumbnail
    """
    mp = IIIFParser.load(*args, **kwargs)
    volume = mp.run()
    return volume.id


def import_annotations_csv(name, raw_path, source, datasource_id, index_root):
    """
    Import annotations from a CSV line
    """
    if raw_path.startswith('/home/data/indexes'):
        raw_path = raw_path[18:]
    raw_path = raw_path.lstrip('/')
    index_path = os.path.join(index_root, raw_path)
    assert os.path.exists(index_path), \
        'Index files path not found for volume {} : {}'.format(name, index_path)

    volume = Element.objects.get(name=name, type=ElementType.Volume)
    datasource = DataSource.objects.get(id=datasource_id, type=MLToolType.Recognizer)

    if source == 'bvmm':
        logger.info("Importing BVMM annotations for volume {}".format(name))
        importer = IndexImporter(index_path, volume, datasource, mask=r'(?:.*/)?([^/]+)_[A-Z]\.idx\.gz')
    elif source == 'gallica':
        logger.info("Importing Gallica annotations for volume {}".format(name))
        importer = GallicaIndexImporter(index_path, volume, datasource)
    else:  # Try anyway
        logger.info("Importing unknown annotations for volume {}".format(name))
        importer = IndexImporter(index_path, volume, datasource)

    importer.run()
