from django.core.management import call_command
from unittest.mock import patch, call
from arkindex.project.tests import FixtureTestCase
import tempfile
import shutil


class TestImportActsCommand(FixtureTestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.surface_mock = patch('arkindex.documents.management.commands.import_acts.SurfaceImporter').start()
        cls.linker_mock = patch('arkindex.documents.management.commands.import_acts.CorpusSurfaceLinker').start()
        cls.xmldir = tempfile.mkdtemp()
        cls.csvdir = tempfile.mkdtemp()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        shutil.rmtree(cls.xmldir)
        shutil.rmtree(cls.csvdir)

    def setUp(self):
        self.surface_mock.reset_mock()
        self.linker_mock.reset_mock()

    def test_run(self):
        call_command(
            'import_acts',
            self.xmldir,
            self.csvdir,
            corpus=self.corpus,
        )
        self.assertEqual(self.surface_mock.call_count, 1)
        self.assertEqual(self.surface_mock.call_args, call(xmlpath=self.xmldir, corpus=self.corpus))
        self.assertEqual(self.surface_mock().run.call_count, 1)
        self.assertEqual(self.surface_mock().run.call_args, call(False))

        self.assertEqual(self.linker_mock.call_count, 1)
        self.assertEqual(self.linker_mock.call_args, call(csvpath=self.csvdir, corpus=self.corpus))
        self.assertEqual(self.linker_mock().run.call_count, 1)
        self.assertEqual(self.linker_mock().run.call_args, call(False))

    def test_dry_run(self):
        call_command(
            'import_acts',
            self.xmldir,
            self.csvdir,
            corpus=self.corpus,
            dry_run=True,
        )
        self.assertEqual(self.surface_mock.call_count, 1)
        self.assertEqual(self.surface_mock.call_args, call(xmlpath=self.xmldir, corpus=self.corpus))
        self.assertEqual(self.surface_mock().run.call_count, 1)
        self.assertEqual(self.surface_mock().run.call_args, call(True))

        self.assertEqual(self.linker_mock.call_count, 1)
        self.assertEqual(self.linker_mock.call_args, call(csvpath=self.csvdir, corpus=self.corpus))
        self.assertEqual(self.linker_mock().run.call_count, 1)
        self.assertEqual(self.linker_mock().run.call_args, call(True))
