from unittest import TestCase
from unittest.mock import patch, mock_open
from pkg_resources import ResolutionError
import arkindex.documents
from arkindex.documents.apps import DocumentsConfig


class TestApps(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.documents = DocumentsConfig('documents', arkindex.documents)

    def test_package_version_from_file(self):
        with patch('arkindex.documents.apps.open', mock_open(read_data='0.42')) as p:
            self.assertEqual(self.documents._package_version('something'), '0.42')
            self.assertTrue(p.called)

    def test_package_version_from_setuptools(self):
        open_mock = patch('arkindex.documents.apps.open').start()
        open_mock.side_effect = OSError
        with patch('arkindex.documents.apps.pkg_resources.get_distribution') as p:
            p().version = '1.3.3.7'
            self.assertEqual(self.documents._package_version('something'), '1.3.3.7')
            self.assertTrue(open_mock.called)
            self.assertTrue(p.called)

    def test_package_version_fail(self):
        open_mock = patch('arkindex.documents.apps.open').start()
        open_mock.side_effect = OSError
        pkg_mock = patch('arkindex.documents.apps.pkg_resources.get_distribution').start()
        pkg_mock.side_effect = ResolutionError
        with self.assertRaises(RuntimeError):
            self.documents._package_version('something')
