from unittest.mock import MagicMock
from django.test import TestCase
from arkindex_common.ml_tool import MLTool, MLToolType
from arkindex.documents.models import DataSource


class TestDataSource(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.ml_tool = MagicMock(spec=MLTool)
        cls.ml_tool.type = MLToolType.Recognizer
        cls.ml_tool.slug = 'muddy'
        cls.ml_tool.version = '0.42'

    def test_from_ml_tool(self):
        ds = DataSource.from_ml_tool(self.ml_tool)
        self.assertEqual(ds.type, MLToolType.Recognizer)
        self.assertEqual(ds.slug, 'muddy')
        self.assertEqual(ds.revision, '0.42')
        self.assertTrue(ds.internal)

    def test_from_ml_tool_exists(self):
        ds1 = DataSource.from_ml_tool(self.ml_tool)
        ds2 = DataSource.from_ml_tool(self.ml_tool)
        self.assertEqual(ds1.id, ds2.id)

    def test_str(self):
        ds = DataSource(
            type=MLToolType.Classifier,
            slug='something',
            revision='1.2.3',
            internal=False,
        )
        self.assertEqual(str(ds), 'something 1.2.3')
