from django.core.exceptions import ValidationError
from arkindex_common.enums import MetaType, EntityType
from arkindex.documents.models import Corpus, Element, ElementType, \
    Entity, EntityRole, EntityLink, MetaData, DataSource, MLToolType
from arkindex.project.tests import FixtureTestCase


class TestSaveEntities(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        source = DataSource.objects.create(type=MLToolType.NER, slug='entity', internal=False)
        cls.corpus1 = Corpus.objects.create(name='corpus 1')
        cls.corpus2 = Corpus.objects.create(name='corpus 2')
        cls.parent = Entity.objects.create(
            name='parent',
            type=EntityType.Organization,
            corpus=cls.corpus1,
            source=source
        )
        cls.child = Entity.objects.create(type=EntityType.Person, corpus=cls.corpus1, name="child", source=source)
        cls.role = EntityRole.objects.create(
            parent_name='organization',
            child_name='person',
            parent_type=EntityType.Organization,
            child_type=EntityType.Person,
            corpus=cls.corpus1,
        )
        cls.link = EntityLink(parent=cls.parent, child=cls.child, role=cls.role)

    def test_parent_type_different(self):
        self.parent.corpus = self.corpus1
        self.child.corpus = self.corpus1
        self.parent.type = EntityType.Person
        self.child.type = EntityType.Person
        with self.assertRaises(ValidationError):
            self.link.save()

    def test_parent_corpus_different(self):
        self.parent.corpus = self.corpus2
        self.child.corpus = self.corpus1
        self.parent.type = EntityType.Organization
        self.child.type = EntityType.Person
        with self.assertRaises(ValidationError):
            self.link.save()

    def test_child_type_different(self):
        self.parent.corpus = self.corpus1
        self.child.corpus = self.corpus1
        self.parent.type = EntityType.Organization
        self.child.type = EntityType.Organization
        with self.assertRaises(ValidationError):
            self.link.save()

    def test_child_corpus_different(self):
        self.parent.corpus = self.corpus1
        self.child.corpus = self.corpus2
        self.parent.type = EntityType.Organization
        self.child.type = EntityType.Person
        with self.assertRaises(ValidationError):
            self.link.save()

    def test_save_entity_in_metadata(self):
        self.parent.corpus = self.corpus2
        element = Element.objects.create(corpus=self.corpus1, type=ElementType.Act, name="element")
        with self.assertRaises(ValidationError):
            MetaData.objects.create(
                name='test 1',
                type=MetaType.Entity,
                value='Blah',
                element=element,
                entity=self.parent,
            )
