from django.views.generic import View
from django.views.generic.detail import SingleObjectMixin
from django.http import HttpResponse
from arkindex.documents.models import Element, ElementType, Act, Page, Corpus
import io
import csv


class DumpActs(SingleObjectMixin, View):
    """
    Dump acts and linked surfaces data for a given volume in CSV format
    """
    def get_queryset(self):
        return Element.objects.filter(type=ElementType.Volume, corpus__in=Corpus.objects.readable(self.request.user))

    def get(self, request, *args, **kwargs):
        volume = self.get_object()
        acts = Act.objects.get_descending(volume.id)
        surfaces = Element.objects.get_descendings(
            *acts.values_list('id', flat=True),
            element_type=ElementType.Surface)
        output = io.StringIO()
        writer = csv.writer(output)
        for act in acts:
            if act.id not in surfaces:
                # Also output acts with no surfaces, with empty columns
                writer.writerow([act.number, act.folio, '', '', '', str(act.id), ''])
            else:
                for surface in surfaces[act.id]:
                    page = Page.objects.filter(zone__image=surface.zone.image).first()
                    writer.writerow([
                        act.number,
                        act.folio,
                        surface.name,
                        page.name,
                        str(surface.zone.polygon.center.y),
                        str(act.id),
                        str(surface.id),
                    ])
        response = HttpResponse(output.getvalue(), content_type="text/csv")
        response['Content-Disposition'] = 'attachment; filename="{}.csv"'.format(volume.name)
        return response
