# Generated by Django 2.1 on 2019-02-26 11:04

from django.db import migrations
from django.db.models.expressions import RawSQL
from itertools import groupby


def deduplicate_and_strip(apps, schema_editor):
    """
    Strips slashes from all existing server URLs and merges them if necessary
    """
    from arkindex.images.models import ImageServer
    all_servers = {
        k: list(v)
        for k, v in groupby(
            ImageServer.objects.annotate(
                # PostgreSQL supports trimming things that are not just spaces,
                # But django.db.models.functions.Trim doesn't.
                trimmed_url=RawSQL("trim(leading '/' from url)", ())
            ).only('id', 'url').order_by('trimmed_url'),
            key=lambda s: s.trimmed_url,
        )
    }
    for trimmed_url, servers in all_servers.items():
        if len(servers) == 1:
            # No duplicate servers, update and continue
            servers[0].url = trimmed_url
            servers[0].save(update_fields={'url'})
            continue

        # There are duplicate servers, find the one that does not have a trailing slash
        original = next(filter(lambda s: s.url == trimmed_url, servers))
        duplicates = list(filter(lambda s: s != original, servers))

        # Merge every other server into this one
        for dupe in duplicates:
            print('Merging {} into {}'.format(dupe.url, original.url))
            dupe.merge_into(original)
            assert not dupe.images.exists()
            dupe.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('images', '0002_update_imageserver'),
    ]

    operations = [
        migrations.RunPython(
            deduplicate_and_strip,
            reverse_code=migrations.RunPython.noop,
            elidable=True,
        ),
    ]
