from rest_framework import permissions


class InternalGroupPermissionMixin(object):

    def has_permission(self, request, view):
        if request.user.is_authenticated and \
                not request.user.is_admin and \
                request.user.is_internal:
            return True

        return super().has_permission(request, view)


class DemoGroupPermissionMixin(object):

    def has_permission(self, request, view):
        if request.user.is_authenticated and not request.user.is_admin:
            # If the user has registered from the demo, or has an unverified email,
            # only allow HEAD, GET and OPTIONS.
            if not request.user.verified_email or request.user.is_demo:
                return super().has_permission(request, view) and request.method in permissions.SAFE_METHODS

        return super().has_permission(request, view)


class AllowAny(InternalGroupPermissionMixin, permissions.AllowAny):
    pass


class IsAuthenticated(InternalGroupPermissionMixin, permissions.IsAuthenticated):
    pass


class IsAuthenticatedOrReadOnly(InternalGroupPermissionMixin, permissions.IsAuthenticatedOrReadOnly):
    pass


class IsAdminUser(InternalGroupPermissionMixin, permissions.IsAdminUser):
    pass


class IsVerified(DemoGroupPermissionMixin, IsAuthenticated):
    pass


class IsVerifiedOrReadOnly(DemoGroupPermissionMixin, IsAuthenticatedOrReadOnly):
    pass
