from unittest import TestCase
from arkindex.project.polygon import Polygon


class TestPolygon(TestCase):
    '''
    Test the low level class
    '''

    def test_base(self):
        a = Polygon([
            [0, 0],
            [100, 0],
            [100, 100],
            [0, 100],
        ])
        self.assertEqual(str(a), 'Polygon((0,0), (100,0), (100,100), (0,100), (0,0))')
        self.assertEqual(a, Polygon.from_coords(0, 0, 100, 100))
        self.assertEqual(a.x, 0)
        self.assertEqual(a.y, 0)
        self.assertEqual(a.width, 100)
        self.assertEqual(a.height, 100)
        self.assertEqual(a.center.x, 50)
        self.assertEqual(a.center.y, 50)
        self.assertEqual(len(a.points), 5)

    def test_dict(self):
        a = Polygon.from_dict({
            'dummy': 'test',
            'polygon': [[0, 0], [0, 1], [1, 1], [1, 0]],
        })
        self.assertEqual(a.x, 0)
        self.assertEqual(a.y, 0)
        self.assertEqual(a.width, 1)
        self.assertEqual(a.height, 1)
        self.assertEqual(len(a.points), 5)

        a = Polygon.from_dict({
            'dummy': 'anything',
            'x': 0,
            'y': 0,
            'width': 1,
            'height': 1,
        })
        self.assertEqual(a.x, 0)
        self.assertEqual(a.y, 0)
        self.assertEqual(a.width, 1)
        self.assertEqual(a.height, 1)
        self.assertEqual(len(a.points), 5)

    def test_str(self):
        a = Polygon.from_str('((0, 0), (0,10),(20, 10),(20,0))')
        self.assertEqual(len(a.points), 5)
        self.assertEqual(a.x, 0)
        self.assertEqual(a.y, 0)
        self.assertEqual(a.width, 20)
        self.assertEqual(a.height, 10)
